/*******************************************************************************
**
**  Module: pps.h
**  Date:   12/02/2003
**  Purpose
**  Description: Header file pps.c
**
**                 Copyright 2002, Spectracom Corporation.
**                          ALL RIGHTS RESERVED
**
** Description:
**     The driver that runs the GPS 1-PPS and the Conditioned 1-PPS.  The PPS
**     API described in RFC 2783 is implemented through ioctls.  In this
**     implementation, the only time format supported is PPS_TMFMT_TSPEC
**
********************************************************************************
**
**     6/28/05 Ported from uClinux (2.4.20) to standard Linux (2.4.26) for the
**             HanSolo project.  [MG]
**
**     10/06/09 Ported from HanSolo project into TSync-PCI driver.  [DHS]
**
*******************************************************************************/
#ifndef PPS_H
#define PPS_H               (1)

/* Includes */
#include <linux/types.h>    /* Needed for specifying data widths. */
#include <linux/timepps.h>  /* PPS-API include */

#ifdef __KERNEL__ /* Don't let userland see these. */
#include <linux/fs.h>        /* Needed to declare ioctl prototype. */
#include <linux/interrupt.h> /* Needed for irq_return_t. */

#include "tsyncpci.h"

/* Defines */

/* Prototypes */
int pps_init    (void);
int pps_init_one(tsyncpci_dev_t *hw);
int pps_exit    (void);
int pps_open    (struct inode *inode, struct file* filp);
int pps_release (struct inode *inode, struct file* filp);
int pps_ioctl   (struct inode*, struct file*, unsigned int, unsigned long);

void pps_isr (int src);

#endif /* __KERNEL */

#endif /* PPS_H */
